from data import Data
import datetime
import time
import sys
import os
import glob
import sets
from dateutil.relativedelta import *
import cdo as cdop
import copy
import numpy
import shutil
import netCDF4 as nc
from common import *
from set_output_name import *

class DataMod(Data):
    """ classe of data for the model
    """
    def duplicate(self):
        """duplicate all atribute execpt the lists"""
        exp2=DataMod("","")
        exp2.set_varname(self.get_varname())
        exp2.set_dataname(self.get_dataname())
        exp2.set_freq(self.get_freq())
        exp2.set_inputdir(self.get_inputdir())
        exp2.set_outputdir(self.get_outputdir())
        exp2.set_nmember(self.get_nmember())
        return exp2

    def set_files(self, date1, date2):
        """create the Filelist and sdates argument with all files
        between date1 and date2 separated by one year
        check if these files exist
        check their type"""
        
        inputdir=self.get_inputdir()
        filelist=[]
        datelist=[]

        filebase=self.get_varname() + "_" + str(date1).replace("-","")
        

        #look for the first file
        filename=glob.glob(inputdir+filebase+'.nc')
        filetype="nc"
        #if no nc file, look for grib
        if len(filename)== 0:
            print("no nc file in"+inputdir)
            return 1
            filename=glob.glob(inputdir+filebase+".grb")
            filetype="grb"
            if len(filename)== 0:
                print "the file %s does not exist " % (inputdir+filebase+".nc")
                print "and the file %s does not exist " % (inputdir+filebase+".grb")
                print "the directory %s must contain .nc or .grb file for the startdate %s" % (inputdir, str(date1))
                return 1
        #store the file
        filelist.append(filename[0])
        datelist.append(date1)
        date1 = date1 + relativedelta(years=+1)
        d=date1
        #loop on the date to find all the files
        while d <= date2:
            filebase=self.get_varname() + "_" + str(d).replace("-","")

            filename=glob.glob(inputdir+filebase+"."+filetype)
            if len(filename)== 0:
                print "the file %s does not exit" % inputdir+filebase+"."+filetype
                print "while the file %s exists"  % inputdir+filelist[0]
                print "all file must have the same type (grb or nc)"
                return 1
            datelist.append(d)
            filelist.append(filename[0])
            d = d + relativedelta(years=+1)

        self.set_filelist(filetype)
        self.set_filelist(filelist)
        self.set_sdates(datelist)
        return 0





    #### to be simplified!!!!!!!!!!!!!!!!!!!!!!!
    def write_output(self):
        cdo=cdop.Cdo()
        cdo.tempfile.tempdir = os.environ['TMPDIR']

        #get the needed variables from the object
        filelist=self.get_filelist()
        sdates=self.get_sdates()
        dataname=self.get_dataname()
        varname=self.get_varname()
        freq=self.get_freq()
        nmember=self.get_nmember()

        name=set_output_name(varname)
        units=name[0]
        unit_long=name[1]
        long_name=name[2]

        #create output directory
        outputdir=self.get_outputdir()+"/"+varname+"/"        
        if (not os.path.isdir(outputdir)):
            os.mkdir(outputdir)
        

        #loop over all file
        for i in range(len(filelist)):
            f=filelist[i]
            ftemp=(f.split("/")[-1])
            ftemp=ftemp.split(".")[0]+"_temp2.nc"
            print(f)
            print(ftemp)
            
            #open input and output
            ncdfin=nc.Dataset(f, "r")
            ncdfout=nc.Dataset(ftemp, mode="w" , format='NETCDF4')

            variables=ncdfin.variables
  
            ##### time ######
            write_time(ncdfout, ncdfin)
           
            ##### longitude - latitude ######
            write_lonlat(ncdfout, ncdfin)

            ##### ensembles ######
            ncdfout.createDimension('ensemble', nmember)

            #### variable #####
            var=ncdfout.createVariable(varname, 'f4', ("time", "ensemble", "latitude", "longitude",))

            if varname in variables:
                varin = ncdfin.variables[varname]
                varaux=varname
            else:
                print("the variable"+varname+" is not in "+f)

            var.data_type = "float" 
            var.units = units
            var.unit_long = unit_long
            var.standard_name = varname 
            var.long_name = long_name

            var[:]=varin

            ##### institution ######
            write_insti(ncdfout,nmember,dataname)
            
            ###### experiment_id ######
            write_id(ncdfout,nmember,dataname)
           
            ###### realization ######
            write_realization(ncdfout,nmember)

            ###### source ######
            write_source(ncdfout,nmember,dataname)
            ncdfout.close()
            fileout=varname+"_"+str(sdates[i]).replace("-","")+".nc"
            shutil.copyfile(ftemp, outputdir+fileout)

            print "file saved:"
            print outputdir+fileout



